#
# Description: Given a Ansible Job Id, check it's status
#

module ManageIQ
  module Automate
    module AutomationManagement
      module AnsibleTower
        module Operations
          module StateMachines
            module Job
              class WaitForCompletion
                JOB_CLASS = 'ManageIQ_Providers_AutomationManager_OrchestrationStack'.freeze
                def initialize(handle = $evm)
                  @handle = handle
                end

                def main
                  check_status(ansible_job)
                end

                private

                def check_status(job)
                  status, reason = job.normalized_live_status
                  case status
                  when 'transient'
                    @handle.root['ae_result'] = 'retry'
                    @handle.root['ae_retry_interval'] = 1.minute
                  when 'failed', 'create_canceled'
                    @handle.root['ae_result'] = 'error'
                    @handle.log(:error, "Job failed for #{job.id} Ansible ID: #{job.ems_ref} reason #{reason}")
                    job.refresh_ems
                  when 'create_complete'
                    @handle.root['ae_result'] = 'ok'
                    job.refresh_ems
                  else
                    @handle.root['ae_result'] = 'error'
                    @handle.log(:error, "Job failed for #{job.id} Ansible ID: #{job.ems_ref} Unknown status #{status} reason #{reason}")
                    job.refresh_ems
                  end
                end

                def ansible_job
                  job_id = @handle.get_state_var(:ansible_job_id)
                  if job_id.nil?
                    @handle.log(:error, 'Ansible job id not found')
                    exit(MIQ_ERROR)
                  end
                  fetch_job(job_id)
                end

                def fetch_job(job_id)
                  job = @handle.vmdb(JOB_CLASS).find(job_id)
                  if job.nil?
                    @handle.log(:error, 'Ansible job with id : #{job_id} not found')
                    exit(MIQ_ERROR)
                  end
                  job
                end
              end
            end
          end
        end
      end
    end
  end
end

ManageIQ::Automate::AutomationManagement::AnsibleTower::Operations::StateMachines::Job::WaitForCompletion.new.main
