class ManageIQ::Providers::Redhat::InfraManager::EventFetcher
  attr_reader :ext_management_system

  def initialize(ems)
    @ext_management_system = ems
  end

  def events(query_options)
    query = {}
    query[:max] = query_options[:max] if query_options[:max]
    query[:from] = query_options[:since] if query_options[:since]
    ext_management_system.with_provider_connection do |ems_service|
      events = ems_service.system_service.events_service.list(query)
      events.each { |event| set_event_name!(event) }
      events
    end
  end

  def set_event_name!(event)
    name = EVENT_CODES[event.code] || "UNKNOWN"
    event.name = name
  end

  EVENT_CODES = {
    # Code   Description
    0     => "UNASSIGNED",
    1     => "VDC_START",
    2     => "VDC_STOP",
    12    => "HOST_FAILURE",
    13    => "HOST_DETECTED",
    14    => "HOST_RECOVER",
    15    => "HOST_MAINTENANCE",
    16    => "HOST_ACTIVATE",
    17    => "HOST_MAINTENANCE_FAILED",
    18    => "HOST_ACTIVATE_FAILED",
    19    => "HOST_RECOVER_FAILED",
    20    => "USER_HOST_START",
    21    => "USER_HOST_STOP",
    22    => "IRS_FAILURE",
    23    => "VDS_LOW_DISK_SPACE",
    24    => "VDS_LOW_DISK_SPACE_ERROR",
    25    => "VDS_NO_SELINUX_ENFORCEMENT",
    26    => "IRS_DISK_SPACE_LOW",
    27    => "VDS_STATUS_CHANGE_FAILED_DUE_TO_STOP_SPM_FAILURE",
    28    => "VDS_PROVISION",
    29    => "USER_ADD_VM_TEMPLATE_SUCCESS",
    30    => "USER_VDC_LOGIN",
    31    => "USER_VDC_LOGOUT",
    32    => "USER_RUN_VM",
    33    => "USER_STOP_VM",
    34    => "USER_ADD_VM",
    35    => "USER_UPDATE_VM",
    36    => "USER_REMOVE_VM",
    37    => "USER_ADD_VM_STARTED",
    38    => "USER_CHANGE_DISK_VM",
    39    => "USER_PAUSE_VM",
    40    => "USER_RESUME_VM",
    41    => "USER_HOST_RESTART",
    42    => "USER_ADD_HOST",
    43    => "USER_UPDATE_HOST",
    44    => "USER_REMOVE_HOST",
    45    => "USER_CREATE_SNAPSHOT",
    46    => "USER_TRY_BACK_TO_SNAPSHOT",
    47    => "USER_RESTORE_FROM_SNAPSHOT",
    48    => "USER_ADD_VM_TEMPLATE",
    49    => "USER_UPDATE_VM_TEMPLATE",
    50    => "USER_REMOVE_VM_TEMPLATE",
    51    => "USER_ADD_VM_TEMPLATE_FINISHED_SUCCESS",
    52    => "USER_ADD_VM_TEMPLATE_FINISHED_FAILURE",
    53    => "USER_ADD_VM_FINISHED_SUCCESS",
    54    => "USER_FAILED_RUN_VM",
    55    => "USER_FAILED_PAUSE_VM",
    56    => "USER_FAILED_STOP_VM",
    57    => "USER_FAILED_ADD_VM",
    58    => "USER_FAILED_UPDATE_VM",
    59    => "USER_FAILED_REMOVE_VM",
    60    => "USER_ADD_VM_FINISHED_FAILURE",
    61    => "VM_DOWN",
    62    => "VM_MIGRATION_START",
    63    => "VM_MIGRATION_DONE",
    64    => "VM_MIGRATION_ABORT",
    65    => "VM_MIGRATION_FAILED",
    66    => "VM_FAILURE",
    67    => "VM_MIGRATION_START_SYSTEM_INITIATED",
    68    => "USER_CREATE_SNAPSHOT_FINISHED_SUCCESS",
    69    => "USER_CREATE_SNAPSHOT_FINISHED_FAILURE",
    70    => "USER_RUN_VM_AS_STATELESS_FINISHED_FAILURE",
    71    => "USER_TRY_BACK_TO_SNAPSHOT_FINISH_SUCCESS",
    72    => "USER_CHANGE_FLOPPY_VM",
    73    => "USER_INITIATED_SHUTDOWN_VM",
    74    => "USER_FAILED_SHUTDOWN_VM",
    75    => "USER_FAILED_CHANGE_FLOPPY_VM",
    76    => "USER_STOPPED_VM_INSTEAD_OF_SHUTDOWN",
    77    => "USER_FAILED_STOPPING_VM_INSTEAD_OF_SHUTDOWN",
    78    => "USER_ADD_DISK_TO_VM",
    79    => "USER_FAILED_ADD_DISK_TO_VM",
    80    => "USER_REMOVE_DISK_FROM_VM",
    81    => "USER_FAILED_REMOVE_DISK_FROM_VM",
    82    => "USER_MOVED_VM",
    83    => "USER_FAILED_MOVE_VM",
    84    => "USER_MOVED_TEMPLATE",
    85    => "USER_FAILED_MOVE_TEMPLATE",
    86    => "USER_COPIED_TEMPLATE",
    87    => "USER_FAILED_COPY_TEMPLATE",
    88    => "USER_UPDATE_VM_DISK",
    89    => "USER_FAILED_UPDATE_VM_DISK",
    90    => "USER_HOST_SHUTDOWN",
    91    => "USER_MOVED_VM_FINISHED_SUCCESS",
    92    => "USER_MOVED_VM_FINISHED_FAILURE",
    93    => "USER_MOVED_TEMPLATE_FINISHED_SUCCESS",
    94    => "USER_MOVED_TEMPLATE_FINISHED_FAILURE",
    95    => "USER_COPIED_TEMPLATE_FINISHED_SUCCESS",
    96    => "USER_COPIED_TEMPLATE_FINISHED_FAILURE",
    97    => "USER_ADD_DISK_TO_VM_FINISHED_SUCCESS",
    98    => "USER_ADD_DISK_TO_VM_FINISHED_FAILURE",
    99    => "USER_TRY_BACK_TO_SNAPSHOT_FINISH_FAILURE",
    100   => "USER_RESTORE_FROM_SNAPSHOT_FINISH_SUCCESS",
    101   => "USER_RESTORE_FROM_SNAPSHOT_FINISH_FAILURE",
    102   => "USER_FAILED_CHANGE_DISK_VM",
    103   => "USER_FAILED_RESUME_VM",
    104   => "USER_FAILED_ADD_HOST",
    105   => "USER_FAILED_UPDATE_HOST",
    106   => "USER_FAILED_REMOVE_HOST",
    107   => "USER_FAILED_HOST_RESTART",
    108   => "USER_FAILED_ADD_VM_TEMPLATE",
    109   => "USER_FAILED_UPDATE_VM_TEMPLATE",
    110   => "USER_FAILED_REMOVE_VM_TEMPLATE",
    111   => "USER_STOP_SUSPENDED_VM",
    112   => "USER_STOP_SUSPENDED_VM_FAILED",
    113   => "USER_REMOVE_VM_FINISHED",
    114   => "USER_VDC_LOGIN_FAILED",
    115   => "USER_FAILED_TRY_BACK_TO_SNAPSHOT",
    116   => "USER_FAILED_RESTORE_FROM_SNAPSHOT",
    117   => "USER_FAILED_CREATE_SNAPSHOT",
    118   => "USER_FAILED_HOST_START",
    119   => "VM_DOWN_ERROR",
    120   => "VM_MIGRATION_FAILED_FROM_TO",
    121   => "SYSTEM_HOST_RESTART",
    122   => "SYSTEM_FAILED_HOST_RESTART",
    123   => "HOST_SLOW_STORAGE_RESPONSE_TIME",
    124   => "VM_IMPORT",
    125   => "VM_IMPORT_FAILED",
    126   => "VM_NOT_RESPONDING",
    127   => "HOST_RUN_IN_NO_KVM_MODE",
    128   => "VM_MIGRATION_TRYING_RERUN",
    129   => "VM_CLEARED",
    130   => "USER_FAILED_HOST_SHUTDOWN",
    131   => "USER_EXPORT_VM",
    132   => "USER_EXPORT_VM_FAILED",
    133   => "USER_EXPORT_TEMPLATE",
    134   => "USER_EXPORT_TEMPLATE_FAILED",
    135   => "TEMPLATE_IMPORT",
    136   => "TEMPLATE_IMPORT_FAILED",
    137   => "USER_FAILED_HOST_STOP",
    138   => "VM_PAUSED_ENOSPC",
    139   => "VM_PAUSED_ERROR",
    140   => "VM_MIGRATION_FAILED_DURING_MOVE_TO_MAINTANANCE",
    141   => "HOST_VERSION_NOT_SUPPORTED_FOR_CLUSTER",
    142   => "VM_SET_TO_UNKNOWN_STATUS",
    143   => "VM_WAS_SET_DOWN_DUE_TO_HOST_REBOOT_OR_MANUAL_FENCE",
    144   => "VM_IMPORT_INFO",
    145   => "VM_BLK_VIRTIO_NO_CACHE",
    146   => "VM_PAUSED_EPERM",
    147   => "VM_SHUTDOWN_FAILED",
    148   => "VM_MEMORY_UNDER_GUARANTEED_VALUE",
    149   => "USER_ADD",
    150   => "USER_INITIATED_RUN_VM",
    151   => "USER_INITIATED_RUN_VM_FAILED",
    152   => "USER_RUN_VM_ON_NON_DEFAULT_HOST",
    153   => "USER_STARTED_VM",
    154   => "VDS_CLUSTER_VERSION_NOT_SUPPORTED",
    155   => "VDS_ARCHITECTURE_NOT_SUPPORTED_FOR_CLUSTER",
    156   => "CPU_TYPE_UNSUPPORTED_IN_THIS_CLUSTER_VERSION",
    157   => "USER_REBOOT_VM",
    158   => "USER_FAILED_REBOOT_VM",
    159   => "USER_FORCE_SELECTED_SPM",
    160   => "USER_ACCOUNT_DISABLED_OR_LOCKED",
    161   => "VM_CANCEL_MIGRATION",
    162   => "VM_CANCEL_MIGRATION_FAILED",
    163   => "VM_STATUS_RESTORED",
    164   => "VM_SET_TICKET",
    165   => "VM_SET_TICKET_FAILED",
    166   => "VM_MIGRATION_NO_VDS_TO_MIGRATE_TO",
    167   => "VM_CONSOLE_CONNECTED",
    168   => "VM_CONSOLE_DISCONNECTED",
    169   => "VM_FAILED_TO_PRESTART_IN_POOL",
    170   => "USER_CREATE_LIVE_SNAPSHOT_FINISHED_FAILURE",
    171   => "USER_RUN_VM_AS_STATELESS_WITH_DISKS_NOT_ALLOWING_SNAPSHOT",
    172   => "USER_REMOVE_VM_FINISHED_WITH_ILLEGAL_DISKS",
    173   => "USER_CREATE_LIVE_SNAPSHOT_NO_MEMORY_FAILURE",
    174   => "VM_IMPORT_FROM_CONFIGURATION_EXECUTED_SUCCESSFULLY",
    175   => "VM_IMPORT_FROM_CONFIGURATION_ATTACH_DISKS_FAILED",
    176   => "VM_BALLOON_DRIVER_ERROR",
    177   => "VM_BALLOON_DRIVER_UNCONTROLLED",
    178   => "VM_MEMORY_NOT_IN_RECOMMENDED_RANGE",
    179   => "USER_INITIATED_RUN_VM_AND_PAUSE",
    180   => "TEMPLATE_IMPORT_FROM_CONFIGURATION_SUCCESS",
    181   => "TEMPLATE_IMPORT_FROM_CONFIGURATION_FAILED",
    182   => "USER_FAILED_ATTACH_USER_TO_VM",
    183   => "USER_ATTACH_TAG_TO_TEMPLATE",
    184   => "USER_ATTACH_TAG_TO_TEMPLATE_FAILED",
    185   => "USER_DETACH_TEMPLATE_FROM_TAG",
    186   => "USER_DETACH_TEMPLATE_FROM_TAG_FAILED",
    187   => "VDS_STORAGE_CONNECTION_FAILED_BUT_LAST_VDS",
    188   => "VDS_STORAGES_CONNECTION_FAILED",
    189   => "VDS_STORAGE_VDS_STATS_FAILED",
    190   => "UPDATE_OVF_FOR_STORAGE_DOMAIN_FAILED",
    191   => "CREATE_OVF_STORE_FOR_STORAGE_DOMAIN_FAILED",
    192   => "CREATE_OVF_STORE_FOR_STORAGE_DOMAIN_INITIATE_FAILED",
    193   => "DELETE_OVF_STORE_FOR_STORAGE_DOMAIN_FAILED",
    194   => "VM_CANCEL_CONVERSION",
    195   => "VM_CANCEL_CONVERSION_FAILED",
    196   => "VM_RECOVERED_FROM_PAUSE_ERROR",
    197   => "SYSTEM_SSH_HOST_RESTART",
    198   => "SYSTEM_FAILED_SSH_HOST_RESTART",
    199   => "USER_UPDATE_OVF_STORE",
    200   => "IMPORTEXPORT_GET_VMS_INFO_FAILED",
    201   => "IRS_DISK_SPACE_LOW_ERROR",
    202   => "IMPORTEXPORT_GET_EXTERNAL_VMS_INFO_FAILED",
    204   => "IRS_HOSTED_ON_HOST",
    205   => "PROVIDER_ADDED",
    206   => "PROVIDER_ADDITION_FAILED",
    207   => "PROVIDER_UPDATED",
    208   => "PROVIDER_UPDATE_FAILED",
    209   => "PROVIDER_REMOVED",
    210   => "PROVIDER_REMOVAL_FAILED",
    213   => "PROVIDER_CERTIFICATE_IMPORTED",
    214   => "PROVIDER_CERTIFICATE_IMPORT_FAILED",
    250   => "USER_UPDATE_VM_CLUSTER_DEFAULT_HOST_CLEARED",
    251   => "USER_REMOVE_VM_TEMPLATE_FINISHED",
    300   => "USER_ADD_VM_POOL",
    301   => "USER_ADD_VM_POOL_FAILED",
    302   => "USER_ADD_VM_POOL_WITH_VMS",
    303   => "USER_ADD_VM_POOL_WITH_VMS_FAILED",
    304   => "USER_REMOVE_VM_POOL",
    305   => "USER_REMOVE_VM_POOL_FAILED",
    306   => "USER_ADD_VM_TO_POOL",
    307   => "USER_ADD_VM_TO_POOL_FAILED",
    308   => "USER_REMOVE_VM_FROM_POOL",
    309   => "USER_REMOVE_VM_FROM_POOL_FAILED",
    310   => "USER_ATTACH_USER_TO_POOL",
    311   => "USER_ATTACH_USER_TO_POOL_FAILED",
    312   => "USER_DETACH_USER_FROM_POOL",
    313   => "USER_DETACH_USER_FROM_POOL_FAILED",
    314   => "USER_UPDATE_VM_POOL",
    315   => "USER_UPDATE_VM_POOL_FAILED",
    316   => "USER_ATTACH_USER_TO_VM_FROM_POOL",
    317   => "USER_ATTACH_USER_TO_VM_FROM_POOL_FAILED",
    318   => "USER_ATTACH_USER_TO_VM_FROM_POOL_FINISHED_SUCCESS",
    319   => "USER_ATTACH_USER_TO_VM_FROM_POOL_FINISHED_FAILURE",
    320   => "USER_ADD_VM_POOL_WITH_VMS_ADD_HOST_FAILED",
    321   => "USER_REMOVE_VM_POOL_INITIATED",
    325   => "USER_REMOVE_ADUSER",
    326   => "USER_FAILED_REMOVE_ADUSER",
    327   => "USER_FAILED_ADD_ADUSER",
    328   => "USER_ATTACH_USER_TO_TIME_LEASED_POOL",
    329   => "USER_ATTACH_USER_TO_TIME_LEASED_POOL_FAILED",
    330   => "USER_DETACH_USER_FROM_TIME_LEASED_POOL",
    331   => "USER_DETACH_USER_FROM_TIME_LEASED_POOL_FAILED",
    332   => "USER_ATTACH_AD_GROUP_TO_TIME_LEASED_POOL",
    333   => "USER_ATTACH_AD_GROUP_TO_TIME_LEASED_POOL_FAILED",
    334   => "USER_DETACH_AD_GROUP_FROM_TIME_LEASED_POOL",
    335   => "USER_DETACH_AD_GROUP_FROM_TIME_LEASED_POOL_FAILED",
    336   => "USER_UPDATE_USER_TO_TIME_LEASED_POOL",
    337   => "USER_UPDATE_USER_TO_TIME_LEASED_POOL_FAILED",
    338   => "USER_UPDATE_AD_GROUP_TO_TIME_LEASED_POOL",
    339   => "USER_UPDATE_AD_GROUP_TO_TIME_LEASED_POOL_FAILED",
    342   => "USER_MERGE_SNAPSHOT",
    343   => "USER_FAILED_MERGE_SNAPSHOT",
    344   => "USER_UPDATE_VM_POOL_WITH_VMS",
    345   => "USER_UPDATE_VM_POOL_WITH_VMS_FAILED",
    346   => "USER_PASSWORD_CHANGED",
    347   => "USER_PASSWORD_CHANGE_FAILED",
    348   => "USER_CLEAR_UNKNOWN_VMS",
    349   => "USER_FAILED_CLEAR_UNKNOWN_VMS",
    350   => "USER_ADD_BOOKMARK",
    351   => "USER_ADD_BOOKMARK_FAILED",
    352   => "USER_UPDATE_BOOKMARK",
    353   => "USER_UPDATE_BOOKMARK_FAILED",
    354   => "USER_REMOVE_BOOKMARK",
    355   => "USER_REMOVE_BOOKMARK_FAILED",
    356   => "USER_MERGE_SNAPSHOT_FINISHED_SUCCESS",
    357   => "USER_MERGE_SNAPSHOT_FINISHED_FAILURE",
    358   => "USER_VM_POOL_MAX_SUBSEQUENT_FAILURES_REACHED",
    359   => "USER_REMOVE_SNAPSHOT_FINISHED_FAILURE_PARTIAL_SNAPSHOT",
    360   => "USER_DETACH_USER_FROM_VM",
    361   => "USER_FAILED_DETACH_USER_FROM_VM",
    362   => "USER_REMOVE_SNAPSHOT_FINISHED_FAILURE_BASE_IMAGE_NOT_FOUND",
    370   => "USER_EXTEND_DISK_SIZE_FAILURE",
    371   => "USER_EXTEND_DISK_SIZE_SUCCESS",
    372   => "USER_EXTEND_DISK_SIZE_UPDATE_VM_FAILURE",
    373   => "USER_REMOVE_DISK_SNAPSHOT",
    374   => "USER_FAILED_REMOVE_DISK_SNAPSHOT",
    375   => "USER_REMOVE_DISK_SNAPSHOT_FINISHED_SUCCESS",
    376   => "USER_REMOVE_DISK_SNAPSHOT_FINISHED_FAILURE",
    377   => "USER_EXTENDED_DISK_SIZE",
    378   => "USER_REGISTER_DISK_FINISHED_SUCCESS",
    379   => "USER_REGISTER_DISK_FINISHED_FAILURE",
    380   => "USER_EXTEND_DISK_SIZE_UPDATE_HOST_FAILURE",
    381   => "USER_REGISTER_DISK_INITIATED",
    400   => "USER_ATTACH_VM_TO_AD_GROUP",
    401   => "USER_ATTACH_VM_TO_AD_GROUP_FAILED",
    402   => "USER_DETACH_VM_TO_AD_GROUP",
    403   => "USER_DETACH_VM_TO_AD_GROUP_FAILED",
    404   => "USER_ATTACH_VM_POOL_TO_AD_GROUP",
    405   => "USER_ATTACH_VM_POOL_TO_AD_GROUP_FAILED",
    406   => "USER_DETACH_VM_POOL_TO_AD_GROUP",
    407   => "USER_DETACH_VM_POOL_TO_AD_GROUP_FAILED",
    408   => "USER_REMOVE_AD_GROUP",
    409   => "USER_REMOVE_AD_GROUP_FAILED",
    430   => "USER_UPDATE_TAG",
    431   => "USER_UPDATE_TAG_FAILED",
    432   => "USER_ADD_TAG",
    433   => "USER_ADD_TAG_FAILED",
    434   => "USER_REMOVE_TAG",
    435   => "USER_REMOVE_TAG_FAILED",
    436   => "USER_ATTACH_TAG_TO_USER",
    437   => "USER_ATTACH_TAG_TO_USER_FAILED",
    438   => "USER_ATTACH_TAG_TO_USER_GROUP",
    439   => "USER_ATTACH_TAG_TO_USER_GROUP_FAILED",
    440   => "USER_ATTACH_TAG_TO_VM",
    441   => "USER_ATTACH_TAG_TO_VM_FAILED",
    442   => "USER_ATTACH_TAG_TO_HOST",
    443   => "USER_ATTACH_TAG_TO_HOST_FAILED",
    444   => "USER_DETACH_HOST_FROM_TAG",
    445   => "USER_DETACH_HOST_FROM_TAG_FAILED",
    446   => "USER_DETACH_VM_FROM_TAG",
    447   => "USER_DETACH_VM_FROM_TAG_FAILED",
    448   => "USER_DETACH_USER_FROM_TAG",
    449   => "USER_DETACH_USER_FROM_TAG_FAILED",
    450   => "USER_DETACH_USER_GROUP_FROM_TAG",
    451   => "USER_DETACH_USER_GROUP_FROM_TAG_FAILED",
    452   => "USER_ATTACH_TAG_TO_USER_EXISTS",
    453   => "USER_ATTACH_TAG_TO_USER_GROUP_EXISTS",
    454   => "USER_ATTACH_TAG_TO_VM_EXISTS",
    455   => "USER_ATTACH_TAG_TO_HOST_EXISTS",
    456   => "USER_LOGGED_IN_VM",
    457   => "USER_LOGGED_OUT_VM",
    458   => "USER_LOCKED_VM",
    459   => "USER_UNLOCKED_VM",
    460   => "USER_DETACH_USER_FROM_TIME_LEASED_POOL_INTERNAL",
    461   => "USER_DETACH_USER_FROM_TIME_LEASED_POOL_FAILED_INTERNAL",
    462   => "USER_DETACH_AD_GROUP_FROM_TIME_LEASED_POOL_INTERNAL",
    463   => "USER_DETACH_AD_GROUP_FROM_TIME_LEASED_POOL_FAILED_INTERNAL",
    467   => "UPDATE_TAGS_VM_DEFAULT_DISPLAY_TYPE",
    468   => "UPDATE_TAGS_VM_DEFAULT_DISPLAY_TYPE_FAILED",
    470   => "USER_ATTACH_VM_POOL_TO_AD_GROUP_INTERNAL",
    471   => "USER_ATTACH_VM_POOL_TO_AD_GROUP_FAILED_INTERNAL",
    472   => "USER_ATTACH_USER_TO_POOL_INTERNAL",
    473   => "USER_ATTACH_USER_TO_POOL_FAILED_INTERNAL",
    493   => "VDS_ALREADY_IN_REQUESTED_STATUS",
    494   => "HOST_MANUAL_FENCE_STATUS",
    495   => "HOST_MANUAL_FENCE_STATUS_FAILED",
    496   => "HOST_FENCE_STATUS",
    497   => "HOST_FENCE_STATUS_FAILED",
    498   => "HOST_APPROVE",
    499   => "HOST_APPROVE_FAILED",
    500   => "HOST_FAILED_TO_RUN_VMS",
    501   => "USER_SUSPEND_VM",
    502   => "USER_FAILED_SUSPEND_VM",
    503   => "USER_SUSPEND_VM_OK",
    504   => "HOST_INSTALL",
    505   => "HOST_INSTALL_FAILED",
    506   => "HOST_INITIATED_RUN_VM",
    507   => "HOST_INITIATED_RUN_VM_FAILED",
    509   => "HOST_INSTALL_IN_PROGRESS",
    510   => "HOST_INSTALL_IN_PROGRESS_WARNING",
    511   => "HOST_INSTALL_IN_PROGRESS_ERROR",
    512   => "USER_SUSPEND_VM_FINISH_SUCCESS",
    513   => "HOST_RECOVER_FAILED_VMS_UNKNOWN",
    514   => "HOST_INITIALIZING",
    515   => "HOST_CPU_LOWER_THAN_CLUSTER",
    516   => "HOST_CPU_RETRIEVE_FAILED",
    517   => "HOST_SET_NONOPERATIONAL",
    518   => "HOST_SET_NONOPERATIONAL_FAILED",
    519   => "HOST_SET_NONOPERATIONAL_NETWORK",
    520   => "USER_ATTACH_USER_TO_VM",
    521   => "USER_SUSPEND_VM_FINISH_FAILURE",
    522   => "HOST_SET_NONOPERATIONAL_DOMAIN",
    523   => "HOST_SET_NONOPERATIONAL_DOMAIN_FAILED",
    524   => "HOST_DOMAIN_DELAY_INTERVAL",
    525   => "AUTO_SUSPEND_VM_FINISH_SUCCESS",
    526   => "AUTO_SUSPEND_VM_FINISH_FAILURE",
    527   => "AUTO_FAILED_SUSPEND_VM",
    528   => "USER_EJECT_VM_DISK",
    529   => "USER_EJECT_VM_FLOPPY",
    530   => "HOST_MANUAL_FENCE_FAILED_CALL_FENCE_SPM",
    531   => "HOST_LOW_MEM",
    532   => "VDS_HIGH_MEM_USE",
    533   => "VDS_HIGH_NETWORK_USE",
    534   => "VDS_HIGH_CPU_USE",
    535   => "VDS_HIGH_SWAP_USE",
    536   => "VDS_LOW_SWAP",
    537   => "VDS_INITIATED_RUN_VM_AS_STATELESS",
    538   => "USER_RUN_VM_AS_STATELESS",
    539   => "VDS_AUTO_FENCE_STATUS",
    540   => "VDS_AUTO_FENCE_STATUS_FAILED",
    541   => "VDS_AUTO_FENCE_FAILED_CALL_FENCE_SPM",
    550   => "VDS_PACKAGES_IN_PROGRESS",
    551   => "VDS_PACKAGES_IN_PROGRESS_WARNING",
    552   => "VDS_PACKAGES_IN_PROGRESS_ERROR",
    555   => "USER_MOVE_TAG",
    556   => "USER_MOVE_TAG_FAILED",
    560   => "VDS_ANSIBLE_INSTALL_STARTED",
    561   => "VDS_ANSIBLE_INSTALL_FINISHED",
    562   => "VDS_ANSIBLE_HOST_REMOVE_STARTED",
    563   => "VDS_ANSIBLE_HOST_REMOVE_FINISHED",
    564   => "VDS_ANSIBLE_HOST_REMOVE_FAILED",
    565   => "VDS_ANSIBLE_HOST_REMOVE_EXECUTION_FAILED",
    600   => "USER_HOST_MAINTENANCE",
    601   => "CPU_FLAGS_NX_IS_MISSING",
    602   => "USER_HOST_MAINTENANCE_MIGRATION_FAILED",
    603   => "HOST_SET_NONOPERATIONAL_IFACE_DOWN",
    604   => "VDS_TIME_DRIFT_ALERT",
    605   => "HOST_USED_AS_PROXY",
    606   => "HOST_REFRESHED_CAPABILITIES",
    607   => "HOST_REFRESH_CAPABILITIES_FAILED",
    608   => "HOST_INTERFACE_HIGH_NETWORK_USE",
    609   => "HOST_INTERFACE_STATE_UP",
    610   => "HOST_INTERFACE_STATE_DOWN",
    611   => "HOST_BOND_SLAVE_STATE_UP",
    612   => "HOST_BOND_SLAVE_STATE_DOWN",
    613   => "FENCE_KDUMP_LISTENER_IS_NOT_ALIVE",
    614   => "KDUMP_FLOW_DETECTED_ON_VDS",
    615   => "KDUMP_FLOW_NOT_DETECTED_ON_VDS",
    616   => "KDUMP_FLOW_FINISHED_ON_VDS",
    617   => "KDUMP_DETECTION_NOT_CONFIGURED_ON_VDS",
    618   => "HOST_REGISTRATION_FAILED_INVALID_CLUSTER",
    619   => "HOST_PROTOCOL_INCOMPATIBLE_WITH_CLUSTER",
    620   => "USER_HOST_MAINTENANCE",
    650   => "USER_UNDO_RESTORE_FROM_SNAPSHOT_START",
    651   => "USER_UNDO_RESTORE_FROM_SNAPSHOT_FINISH_SUCCESS",
    652   => "USER_UNDO_RESTORE_FROM_SNAPSHOT_FINISH_FAILURE",
    700   => "DISK_ALIGNMENT_SCAN_START",
    701   => "DISK_ALIGNMENT_SCAN_FAILURE",
    702   => "DISK_ALIGNMENT_SCAN_SUCCESS",
    800   => "IMAGES_SYNCRONIZER_DESKTOP_NOT_EXIST_IN_VDC",
    801   => "IMAGES_SYNCRONIZER_TEMPLATE_NOT_EXIST_IMAGE_EXIST",
    802   => "IMAGES_SYNCRONIZER_SNAPSHOT_NOT_EXIST_IN_VDC",
    803   => "IMAGES_SYNCRONIZER_SNAPSHOTS_NOT_ATTACHED_TO_VM_IN_VDC",
    804   => "IMAGES_SYNCRONIZER_TEMPLATE_NOT_EXIST_IN_VDC",
    805   => "IMAGES_SYNCRONIZER_DESKTOP_NOT_EXIST_IN_IRS",
    806   => "IMAGES_SYNCRONIZER_SNAPSHOT_NOT_EXIST_IN_IRS",
    807   => "IMAGES_SYNCRONIZER_DESKTOP_WITHOUT_TEMPLATE_VDC",
    808   => "IMAGES_SYNCRONIZER_IMAGE_TEMPLATE_NOT_EXIST",
    809   => "USER_ADD_HOST_GROUP",
    810   => "USER_ADD_HOST_GROUP_FAILED",
    811   => "USER_UPDATE_HOST_GROUP",
    812   => "USER_UPDATE_HOST_GROUP_FAILED",
    813   => "USER_REMOVE_HOST_GROUP",
    814   => "USER_REMOVE_HOST_GROUP_FAILED",
    815   => "USER_VDC_LOGOUT_FAILED",
    816   => "MAC_POOL_EMPTY",
    817   => "CERTIFICATE_FILE_NOT_FOUND",
    818   => "RUN_VM_FAILED",
    819   => "HOST_REGISTER_ERROR_UPDATING_HOST",
    820   => "HOST_REGISTER_ERROR_UPDATING_HOST_ALL_TAKEN",
    821   => "HOST_REGISTER_HOST_IS_ACTIVE",
    822   => "HOST_REGISTER_ERROR_UPDATING_NAME",
    823   => "HOST_REGISTER_ERROR_UPDATING_NAMES_ALL_TAKEN",
    824   => "HOST_REGISTER_NAME_IS_ACTIVE",
    825   => "HOST_REGISTER_AUTO_APPROVE_PATTERN",
    826   => "HOST_REGISTER_FAILED",
    827   => "HOST_REGISTER_EXISTING_HOST_UPDATE_FAILED",
    828   => "HOST_REGISTER_SUCCEEDED",
    829   => "VM_MIGRATION_ON_CONNECT_CHECK_FAILED",
    830   => "VM_MIGRATION_ON_CONNECT_CHECK_SUCCEEDED",
    831   => "USER_DEDICATE_VM_TO_POWERCLIENT",
    832   => "USER_DEDICATE_VM_TO_POWERCLIENT_FAILED",
    833   => "MAC_ADDRESS_IS_IN_USE",
    834   => "VDS_REGISTER_EMPTY_ID",
    835   => "SYSTEM_UPDATE_HOST_GROUP",
    836   => "SYSTEM_UPDATE_HOST_GROUP_FAILED",
    837   => "MAC_ADDRESSES_POOL_NOT_INITIALIZED",
    838   => "MAC_ADDRESS_IS_IN_USE_UNPLUG",
    839   => "HOST_AVAILABLE_UPDATES_FAILED",
    840   => "HOST_UPGRADE_STARTED",
    841   => "HOST_UPGRADE_FAILED",
    842   => "HOST_UPGRADE_FINISHED",
    843   => "HOST_UPDATES_ARE_AVAILABLE_WITH_PACKAGES",
    844   => "HOST_UPDATES_ARE_AVAILABLE",
    845   => "HOST_CERTIFICATION_IS_ABOUT_TO_EXPIRE",
    846   => "ENGINE_CERTIFICATION_HAS_EXPIRED",
    847   => "ENGINE_CERTIFICATION_IS_ABOUT_TO_EXPIRE",
    848   => "ENGINE_CA_CERTIFICATION_HAS_EXPIRED",
    849   => "ENGINE_CA_CERTIFICATION_IS_ABOUT_TO_EXPIRE",
    850   => "USER_ADD_PERMISSION",
    851   => "USER_ADD_PERMISSION_FAILED",
    852   => "USER_REMOVE_PERMISSION",
    853   => "USER_REMOVE_PERMISSION_FAILED",
    854   => "USER_ADD_ROLE",
    855   => "USER_ADD_ROLE_FAILED",
    856   => "USER_UPDATE_ROLE",
    857   => "USER_UPDATE_ROLE_FAILED",
    858   => "USER_REMOVE_ROLE",
    859   => "USER_REMOVE_ROLE_FAILED",
    860   => "USER_ATTACHED_ACTION_GROUP_TO_ROLE",
    861   => "USER_ATTACHED_ACTION_GROUP_TO_ROLE_FAILED",
    862   => "USER_DETACHED_ACTION_GROUP_FROM_ROLE",
    863   => "USER_DETACHED_ACTION_GROUP_FROM_ROLE_FAILED",
    864   => "USER_ADD_ROLE_WITH_ACTION_GROUP",
    865   => "USER_ADD_ROLE_WITH_ACTION_GROUP_FAILED",
    866   => "USER_ADD_SYSTEM_PERMISSION",
    867   => "USER_ADD_SYSTEM_PERMISSION_FAILED",
    868   => "USER_REMOVE_SYSTEM_PERMISSION",
    869   => "USER_REMOVE_SYSTEM_PERMISSION_FAILED",
    870   => "USER_ADD_PROFILE",
    871   => "USER_ADD_PROFILE_FAILED",
    872   => "USER_UPDATE_PROFILE",
    873   => "USER_UPDATE_PROFILE_FAILED",
    874   => "USER_REMOVE_PROFILE",
    875   => "USER_REMOVE_PROFILE_FAILED",
    876   => "HOST_CERTIFICATION_IS_INVALID",
    877   => "HOST_CERTIFICATION_HAS_EXPIRED",
    878   => "ENGINE_CERTIFICATION_IS_ABOUT_TO_EXPIRE_ALERT",
    879   => "HOST_CERTIFICATION_IS_ABOUT_TO_EXPIRE_ALERT",
    880   => "HOST_CERTIFICATION_ENROLLMENT_STARTED",
    881   => "HOST_CERTIFICATION_ENROLLMENT_FINISHED",
    882   => "HOST_CERTIFICATION_ENROLLMENT_FAILED",
    883   => "ENGINE_CA_CERTIFICATION_IS_ABOUT_TO_EXPIRE_ALERT",
    884   => "HOST_AVAILABLE_UPDATES_STARTED",
    885   => "HOST_AVAILABLE_UPDATES_FINISHED",
    886   => "HOST_AVAILABLE_UPDATES_PROCESS_IS_ALREADY_RUNNING",
    887   => "HOST_AVAILABLE_UPDATES_SKIPPED_UNSUPPORTED_STATUS",
    890   => "HOST_UPGRADE_FINISHED_MANUAL_HA",
    900   => "AD_COMPUTER_ACCOUNT_SUCCEEDED",
    901   => "AD_COMPUTER_ACCOUNT_FAILED",
    918   => "USER_FORCE_REMOVE_STORAGE_POOL",
    919   => "USER_FORCE_REMOVE_STORAGE_POOL_FAILED",
    920   => "NETWORK_ATTACH_NETWORK_TO_HOST",
    921   => "NETWORK_ATTACH_NETWORK_TO_HOST_FAILED",
    922   => "NETWORK_DETACH_NETWORK_FROM_HOST",
    923   => "NETWORK_DETACH_NETWORK_FROM_HOST_FAILED",
    924   => "NETWORK_ADD_BOND",
    925   => "NETWORK_ADD_BOND_FAILED",
    926   => "NETWORK_REMOVE_BOND",
    927   => "NETWORK_REMOVE_BOND_FAILED",
    928   => "NETWORK_HOST_NETWORK_MATCH_CLUSTER",
    929   => "NETWORK_HOST_NETWORK_NOT_MATCH_CLUSTER",
    930   => "NETWORK_REMOVE_VM_INTERFACE",
    931   => "NETWORK_REMOVE_VM_INTERFACE_FAILED",
    932   => "NETWORK_ADD_VM_INTERFACE",
    933   => "NETWORK_ADD_VM_INTERFACE_FAILED",
    934   => "NETWORK_UPDATE_VM_INTERFACE",
    935   => "NETWORK_UPDATE_VM_INTERFACE_FAILED",
    936   => "NETWORK_ADD_TEMPLATE_INTERFACE",
    937   => "NETWORK_ADD_TEMPLATE_INTERFACE_FAILED",
    938   => "NETWORK_REMOVE_TEMPLATE_INTERFACE",
    939   => "NETWORK_REMOVE_TEMPLATE_INTERFACE_FAILED",
    940   => "NETWORK_UPDATE_TEMPLATE_INTERFACE",
    941   => "NETWORK_UPDATE_TEMPLATE_INTERFACE_FAILED",
    942   => "NETWORK_ADD_NETWORK",
    943   => "NETWORK_ADD_NETWORK_FAILED",
    944   => "NETWORK_REMOVE_NETWORK",
    945   => "NETWORK_REMOVE_NETWORK_FAILED",
    946   => "NETWORK_ATTACH_NETWORK_TO_HOST_GROUP",
    947   => "NETWORK_ATTACH_NETWORK_TO_HOST_GROUP_FAILED",
    948   => "NETWORK_DETACH_NETWORK_TO_HOST_GROUP",
    949   => "NETWORK_DETACH_NETWORK_TO_HOST_GROUP_FAILED",
    950   => "USER_ADD_STORAGE_POOL",
    951   => "USER_ADD_STORAGE_POOL_FAILED",
    952   => "USER_UPDATE_STORAGE_POOL",
    953   => "USER_UPDATE_STORAGE_POOL_FAILED",
    954   => "USER_REMOVE_STORAGE_POOL",
    955   => "USER_REMOVE_STORAGE_POOL_FAILED",
    956   => "USER_ADD_STORAGE_DOMAIN",
    957   => "USER_ADD_STORAGE_DOMAIN_FAILED",
    958   => "USER_UPDATE_STORAGE_DOMAIN",
    959   => "USER_UPDATE_STORAGE_DOMAIN_FAILED",
    960   => "USER_REMOVE_STORAGE_DOMAIN",
    961   => "USER_REMOVE_STORAGE_DOMAIN_FAILED",
    962   => "USER_ATTACH_STORAGE_DOMAIN_TO_POOL",
    963   => "USER_ATTACH_STORAGE_DOMAIN_TO_POOL_FAILED",
    964   => "USER_DETACH_STORAGE_DOMAIN_FROM_POOL",
    965   => "USER_DETACH_STORAGE_DOMAIN_FROM_POOL_FAILED",
    966   => "USER_ACTIVATED_STORAGE_DOMAIN",
    967   => "USER_ACTIVATE_STORAGE_DOMAIN_FAILED",
    968   => "USER_DEACTIVATED_STORAGE_DOMAIN",
    969   => "USER_DEACTIVATE_STORAGE_DOMAIN_FAILED",
    970   => "SYSTEM_DEACTIVATED_STORAGE_DOMAIN",
    971   => "SYSTEM_DEACTIVATE_STORAGE_DOMAIN_FAILED",
    972   => "USER_EXTENDED_STORAGE_DOMAIN",
    973   => "USER_EXTENDED_STORAGE_DOMAIN_FAILED",
    974   => "USER_REMOVE_VG",
    975   => "USER_REMOVE_VG_FAILED",
    976   => "USER_ACTIVATE_STORAGE_POOL",
    977   => "USER_ACTIVATE_STORAGE_POOL_FAILED",
    978   => "SYSTEM_FAILED_CHANGE_STORAGE_POOL_STATUS",
    979   => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_NO_HOST_FOR_SPM",
    980   => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_PROBLEMATIC",
    981   => "USER_FORCE_REMOVE_STORAGE_DOMAIN",
    982   => "USER_FORCE_REMOVE_STORAGE_DOMAIN_FAILED",
    983   => "RECONSTRUCT_MASTER_FAILED_NO_MASTER",
    984   => "RECONSTRUCT_MASTER_DONE",
    985   => "RECONSTRUCT_MASTER_FAILED",
    986   => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_PROBLEMATIC_SEARCHING_NEW_SPM",
    987   => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_PROBLEMATIC_WITH_ERROR",
    988   => "USER_CONNECT_HOSTS_TO_LUN_FAILED",
    989   => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_PROBLEMATIC_FROM_NON_OPERATIONAL",
    990   => "SYSTEM_MASTER_DOMAIN_NOT_IN_SYNC",
    991   => "RECOVERY_STORAGE_POOL",
    992   => "RECOVERY_STORAGE_POOL_FAILED",
    993   => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_RESET_IRS",
    994   => "CONNECT_STORAGE_SERVERS_FAILED",
    995   => "CONNECT_STORAGE_POOL_FAILED",
    996   => "STORAGE_DOMAIN_ERROR",
    997   => "REFRESH_REPOSITORY_IMAGE_LIST_FAILED",
    998   => "REFRESH_REPOSITORY_IMAGE_LIST_SUCCEEDED",
    999   => "STORAGE_ALERT_VG_METADATA_CRITICALLY_FULL",
    1000  => "STORAGE_ALERT_SMALL_VG_METADATA",
    1001  => "USER_RUN_VM_FAILURE_STATELESS_SNAPSHOT_LEFT",
    1002  => "USER_ATTACH_STORAGE_DOMAINS_TO_POOL",
    1003  => "USER_ATTACH_STORAGE_DOMAINS_TO_POOL_FAILED",
    1004  => "STORAGE_DOMAIN_TASKS_ERROR",
    1005  => "UPDATE_OVF_FOR_STORAGE_POOL_FAILED",
    1006  => "UPGRADE_STORAGE_POOL_ENCOUNTERED_PROBLEMS",
    1007  => "REFRESH_REPOSITORY_IMAGE_LIST_INCOMPLETE",
    1008  => "NUMBER_OF_LVS_ON_STORAGE_DOMAIN_EXCEEDED_THRESHOLD",
    1009  => "USER_DEACTIVATE_STORAGE_DOMAIN_OVF_UPDATE_INCOMPLETE",
    1010  => "RELOAD_CONFIGURATIONS_SUCCESS",
    1011  => "RELOAD_CONFIGURATIONS_FAILURE",
    1012  => "NETWORK_INTERFACE_PLUGGED_INTO_VM",
    1013  => "NETWORK_ACTIVATE_VM_INTERFACE_FAILURE",
    1014  => "NETWORK_DEACTIVATE_VM_INTERFACE_SUCCESS",
    1015  => "NETWORK_DEACTIVATE_VM_INTERFACE_FAILURE",
    1016  => "UPDATE_FOR_OVF_STORES_FAILED",
    1017  => "RETRIEVE_OVF_STORE_FAILED",
    1018  => "OVF_STORE_DOES_NOT_EXISTS",
    1019  => "UPDATE_DESCRIPTION_FOR_DISK_FAILED",
    1020  => "UPDATE_DESCRIPTION_FOR_DISK_SKIPPED_SINCE_STORAGE_DOMAIN_NOT_ACTIVE",
    1022  => "USER_REFRESH_LUN_STORAGE_DOMAIN",
    1023  => "USER_REFRESH_LUN_STORAGE_DOMAIN_FAILED",
    1024  => "USER_REFRESH_LUN_STORAGE_DIFFERENT_SIZE_DOMAIN_FAILED",
    1025  => "VM_PAUSED",
    1026  => "FAILED_TO_STORE_ENTIRE_DISK_FIELD_IN_DISK_DESCRIPTION_METADATA",
    1027  => "FAILED_TO_STORE_ENTIRE_DISK_FIELD_AND_REST_OF_FIELDS_IN_DISK_DESCRIPTION_METADATA",
    1028  => "FAILED_TO_STORE_DISK_FIELDS_IN_DISK_DESCRIPTION_METADATA",
    1029  => "STORAGE_DOMAIN_MOVED_TO_MAINTENANCE",
    1030  => "USER_DEACTIVATED_LAST_MASTER_STORAGE_DOMAIN",
    1031  => "TRANSFER_IMAGE_INITIATED",
    1032  => "TRANSFER_IMAGE_SUCCEEDED",
    1033  => "TRANSFER_IMAGE_CANCELLED",
    1034  => "TRANSFER_IMAGE_FAILED",
    1035  => "TRANSFER_IMAGE_TEARDOWN_FAILED",
    1036  => "USER_SCAN_STORAGE_DOMAIN_FOR_UNREGISTERED_DISKS",
    1037  => "USER_SCAN_STORAGE_DOMAIN_FOR_UNREGISTERED_DISKS_FAILED",
    1038  => "UPLOAD_IMAGE_NETWORK_ERROR",
    1039  => "LUNS_BROKE_SD_PASS_DISCARD_SUPPORT",
    1040  => "DISKS_WITH_ILLEGAL_PASS_DISCARD_EXIST",
    1041  => "USER_REMOVE_DEVICE_FROM_STORAGE_DOMAIN_FAILED",
    1042  => "USER_REMOVE_DEVICE_FROM_STORAGE_DOMAIN",
    1043  => "USER_REMOVE_DEVICE_FROM_STORAGE_DOMAIN_STARTED",
    1044  => "ILLEGAL_STORAGE_DOMAIN_DISCARD_AFTER_DELETE",
    1045  => "LUNS_BROKE_SD_DISCARD_AFTER_DELETE_SUPPORT",
    1046  => "STORAGE_DOMAINS_COULD_NOT_BE_SYNCED",
    1047  => "UPLOAD_IMAGE_CLIENT_ERROR",
    1048  => "DIRECT_LUNS_COULD_NOT_BE_SYNCED",
    1049  => "DOWNLOAD_IMAGE_NETWORK_ERROR",
    1052  => "OVF_STORES_UPDATE_IGNORED",
    1098  => "NETWORK_UPDATE_DISPLAY_FOR_HOST_WITH_ACTIVE_VM",
    1099  => "NETWORK_UPDATE_DISPLAY_FOR_CLUSTER_WITH_ACTIVE_VM",
    1100  => "NETWORK_UPDATE_DISPLAY_TO_HOST_GROUP",
    1101  => "NETWORK_UPDATE_DISPLAY_TO_HOST_GROUP_FAILED",
    1102  => "NETWORK_UPDATE_NETWORK_TO_HOST_INTERFACE",
    1103  => "NETWORK_UPDATE_NETWORK_TO_HOST_INTERFACE_FAILED",
    1104  => "NETWORK_COMMINT_NETWORK_CHANGES",
    1105  => "NETWORK_COMMINT_NETWORK_CHANGES_FAILED",
    1106  => "NETWORK_HOST_USING_WRONG_CLUSER_VLAN",
    1107  => "NETWORK_HOST_MISSING_CLUSER_VLAN",
    1108  => "VDS_NETWORK_MTU_DIFFER_FROM_LOGICAL_NETWORK",
    1109  => "BRIDGED_NETWORK_OVER_MULTIPLE_INTERFACES",
    1110  => "VDS_NETWORKS_OUT_OF_SYNC",
    1111  => "VM_MIGRATION_FAILED_DURING_MOVE_TO_MAINTENANCE_NO_DESTINATION_VDS",
    1112  => "NETWORK_UPDTAE_NETWORK_ON_CLUSTER",
    1113  => "NETWORK_UPDTAE_NETWORK_ON_CLUSTER_FAILED",
    1114  => "NETWORK_UPDATE_NETWORK",
    1115  => "NETWORK_UPDATE_NETWORK_FAILED",
    1116  => "NETWORK_UPDATE_VM_INTERFACE_LINK_UP",
    1117  => "NETWORK_UPDATE_VM_INTERFACE_LINK_DOWN",
    1118  => "INVALID_BOND_INTERFACE_FOR_MANAGEMENT_NETWORK_CONFIGURATION",
    1119  => "VLAN_ID_MISMATCH_FOR_MANAGEMENT_NETWORK_CONFIGURATION",
    1120  => "SETUP_NETWORK_FAILED_FOR_MANAGEMENT_NETWORK_CONFIGURATION",
    1121  => "PERSIST_NETWORK_FAILED_FOR_MANAGEMENT_NETWORK",
    1122  => "ADD_VNIC_PROFILE",
    1123  => "ADD_VNIC_PROFILE_FAILED",
    1124  => "UPDATE_VNIC_PROFILE",
    1125  => "UPDATE_VNIC_PROFILE_FAILED",
    1126  => "REMOVE_VNIC_PROFILE",
    1127  => "REMOVE_VNIC_PROFILE_FAILED",
    1128  => "NETWORK_WITHOUT_INTERFACES",
    1129  => "VNIC_PROFILE_UNSUPPORTED_FEATURES",
    1131  => "REMOVE_NETWORK_BY_LABEL_FAILED",
    1132  => "LABEL_NETWORK",
    1133  => "LABEL_NETWORK_FAILED",
    1134  => "UNLABEL_NETWORK",
    1135  => "UNLABEL_NETWORK_FAILED",
    1136  => "LABEL_NIC",
    1137  => "LABEL_NIC_FAILED",
    1138  => "UNLABEL_NIC",
    1139  => "UNLABEL_NIC_FAILED",
    1140  => "SUBNET_REMOVED",
    1141  => "SUBNET_REMOVAL_FAILED",
    1142  => "SUBNET_ADDED",
    1143  => "SUBNET_ADDITION_FAILED",
    1144  => "CONFIGURE_NETWORK_BY_LABELS_WHEN_CHANGING_CLUSTER_FAILED",
    1145  => "PERSIST_NETWORK_ON_HOST",
    1146  => "PERSIST_NETWORK_ON_HOST_FINISHED",
    1147  => "PERSIST_NETWORK_ON_HOST_FAILED",
    1148  => "MULTI_UPDATE_NETWORK_NOT_POSSIBLE",
    1149  => "REMOVE_PORT_FROM_EXTERNAL_PROVIDER_FAILED",
    1150  => "IMPORTEXPORT_EXPORT_VM",
    1151  => "IMPORTEXPORT_EXPORT_VM_FAILED",
    1152  => "IMPORTEXPORT_IMPORT_VM",
    1153  => "IMPORTEXPORT_IMPORT_VM_FAILED",
    1154  => "IMPORTEXPORT_REMOVE_TEMPLATE",
    1155  => "IMPORTEXPORT_REMOVE_TEMPLATE_FAILED",
    1156  => "IMPORTEXPORT_EXPORT_TEMPLATE",
    1157  => "IMPORTEXPORT_EXPORT_TEMPLATE_FAILED",
    1158  => "IMPORTEXPORT_IMPORT_TEMPLATE",
    1159  => "IMPORTEXPORT_IMPORT_TEMPLATE_FAILED",
    1160  => "IMPORTEXPORT_REMOVE_VM",
    1161  => "IMPORTEXPORT_REMOVE_VM_FAILED",
    1162  => "IMPORTEXPORT_STARTING_EXPORT_VM",
    1163  => "IMPORTEXPORT_STARTING_IMPORT_TEMPLATE",
    1164  => "IMPORTEXPORT_STARTING_EXPORT_TEMPLATE",
    1165  => "IMPORTEXPORT_STARTING_IMPORT_VM",
    1166  => "IMPORTEXPORT_STARTING_REMOVE_TEMPLATE",
    1167  => "IMPORTEXPORT_STARTING_REMOVE_VM",
    1168  => "IMPORTEXPORT_FAILED_TO_IMPORT_VM",
    1169  => "IMPORTEXPORT_FAILED_TO_IMPORT_TEMPLATE",
    1170  => "IMPORTEXPORT_IMPORT_TEMPLATE_INVALID_INTERFACES",
    1171  => "USER_ACCOUNT_PASSWORD_EXPIRED",
    1172  => "AUTH_FAILED_INVALID_CREDENTIALS",
    1173  => "AUTH_FAILED_CLOCK_SKEW_TOO_GREAT",
    1174  => "AUTH_FAILED_NO_KDCS_FOUND",
    1175  => "AUTH_FAILED_DNS_ERROR",
    1176  => "AUTH_FAILED_OTHER",
    1177  => "AUTH_FAILED_DNS_COMMUNICATION_ERROR",
    1178  => "AUTH_FAILED_CONNECTION_TIMED_OUT",
    1179  => "AUTH_FAILED_WRONG_REALM",
    1180  => "AUTH_FAILED_CONNECTION_ERROR",
    1181  => "AUTH_FAILED_CANNOT_FIND_LDAP_SERVER_FOR_DOMAIN",
    1182  => "AUTH_FAILED_NO_USER_INFORMATION_WAS_FOUND",
    1183  => "AUTH_FAILED_CLIENT_NOT_FOUND_IN_KERBEROS_DATABASE",
    1184  => "AUTH_FAILED_INTERNAL_KERBEROS_ERROR",
    1185  => "USER_ACCOUNT_EXPIRED",
    1186  => "IMPORTEXPORT_NO_PROXY_HOST_AVAILABLE_IN_DC",
    1187  => "IMPORTEXPORT_HOST_CANNOT_SERVE_AS_PROXY",
    1188  => "IMPORTEXPORT_PARTIAL_VM_DISKS_NOT_EXISTS",
    1189  => "IMPORTEXPORT_IMPORT_VM_FAILED_UPDATING_OVF",
    1190  => "USER_RESTORE_FROM_SNAPSHOT_START",
    1191  => "VM_DISK_ALREADY_CHANGED",
    1192  => "VM_DISK_ALREADY_EJECTED",
    1193  => "IMPORTEXPORT_STARTING_CONVERT_VM",
    1194  => "IMPORTEXPORT_CONVERT_FAILED",
    1195  => "IMPORTEXPORT_CANNOT_GET_OVF",
    1196  => "IMPORTEXPORT_INVALID_OVF",
    1197  => "IMPORTEXPORT_PARTIAL_TEMPLATE_DISKS_NOT_EXISTS",
    1200  => "VM_RENAMED",
    1201  => "UPDATE_HOST_NIC_VFS_CONFIG",
    1202  => "UPDATE_HOST_NIC_VFS_CONFIG_FAILED",
    1203  => "ADD_VFS_CONFIG_NETWORK",
    1204  => "ADD_VFS_CONFIG_NETWORK_FAILED",
    1205  => "REMOVE_VFS_CONFIG_NETWORK",
    1206  => "REMOVE_VFS_CONFIG_NETWORK_FAILED",
    1207  => "ADD_VFS_CONFIG_LABEL",
    1208  => "ADD_VFS_CONFIG_LABEL_FAILED",
    1209  => "REMOVE_VFS_CONFIG_LABEL",
    1210  => "REMOVE_VFS_CONFIG_LABEL_FAILED",
    1211  => "USER_REDUCE_DOMAIN_DEVICES_STARTED",
    1212  => "USER_REDUCE_DOMAIN_DEVICES_FAILED_METADATA_DEVICES",
    1213  => "USER_REDUCE_DOMAIN_DEVICES_FAILED",
    1214  => "USER_REDUCE_DOMAIN_DEVICES_SUCCEEDED",
    1215  => "USER_REDUCE_DOMAIN_DEVICES_FAILED_NO_FREE_SPACE",
    1216  => "USER_REDUCE_DOMAIN_DEVICES_FAILED_TO_GET_DOMAIN_INFO",
    1217  => "CANNOT_IMPORT_VM_WITH_LEASE_COMPAT_VERSION",
    1218  => "CANNOT_IMPORT_VM_WITH_LEASE_STORAGE_DOMAIN",
    1219  => "FAILED_DETERMINE_STORAGE_DOMAIN_METADATA_DEVICES",
    1220  => "HOT_PLUG_LEASE_FAILED",
    1221  => "HOT_UNPLUG_LEASE_FAILED",
    1223  => "IMPORTEXPORT_STARTING_EXPORT_VM_TO_OVA",
    1224  => "IMPORTEXPORT_EXPORT_VM_TO_OVA",
    1225  => "IMPORTEXPORT_EXPORT_VM_TO_OVA_FAILED",
    1300  => "NUMA_ADD_VM_NUMA_NODE_SUCCESS",
    1301  => "NUMA_ADD_VM_NUMA_NODE_FAILED",
    1310  => "NUMA_UPDATE_VM_NUMA_NODE_SUCCESS",
    1311  => "NUMA_UPDATE_VM_NUMA_NODE_FAILED",
    1320  => "NUMA_REMOVE_VM_NUMA_NODE_SUCCESS",
    1321  => "NUMA_REMOVE_VM_NUMA_NODE_FAILED",
    1322  => "USER_ADD_VM_TEMPLATE_CREATE_TEMPLATE_FAILURE",
    1323  => "USER_ADD_VM_TEMPLATE_ASSIGN_ILLEGAL_FAILURE",
    1324  => "USER_ADD_VM_TEMPLATE_SEAL_FAILURE",
    1325  => "USER_SPARSIFY_IMAGE_START",
    1326  => "USER_SPARSIFY_IMAGE_FINISH_SUCCESS",
    1327  => "USER_SPARSIFY_IMAGE_FINISH_FAILURE",
    1328  => "USER_AMEND_IMAGE_START",
    1329  => "USER_AMEND_IMAGE_FINISH_SUCCESS",
    1330  => "USER_AMEND_IMAGE_FINISH_FAILURE",
    1400  => "ENTITY_RENAMED_INTERNALLY",
    1401  => "USER_LOGIN_ON_BEHALF",
    1402  => "USER_LOGIN_ON_BEHALF_FAILED",
    2000  => "USER_HOTPLUG_DISK",
    2001  => "USER_FAILED_HOTPLUG_DISK",
    2002  => "USER_HOTUNPLUG_DISK",
    2003  => "USER_FAILED_HOTUNPLUG_DISK",
    2004  => "USER_COPIED_DISK",
    2005  => "USER_FAILED_COPY_DISK",
    2006  => "USER_COPIED_DISK_FINISHED_SUCCESS",
    2007  => "USER_COPIED_DISK_FINISHED_FAILURE",
    2008  => "USER_MOVED_DISK",
    2009  => "USER_FAILED_MOVED_VM_DISK",
    2010  => "USER_MOVED_DISK_FINISHED_SUCCESS",
    2011  => "USER_MOVED_DISK_FINISHED_FAILURE",
    2012  => "USER_FINISHED_REMOVE_DISK_NO_DOMAIN",
    2013  => "USER_FINISHED_FAILED_REMOVE_DISK_NO_DOMAIN",
    2014  => "USER_FINISHED_REMOVE_DISK",
    2015  => "USER_FINISHED_FAILED_REMOVE_DISK",
    2016  => "USER_ATTACH_DISK_TO_VM",
    2017  => "USER_FAILED_ATTACH_DISK_TO_VM",
    2018  => "USER_DETACH_DISK_FROM_VM",
    2019  => "USER_FAILED_DETACH_DISK_FROM_VM",
    2020  => "USER_ADD_DISK",
    2021  => "USER_ADD_DISK_FINISHED_SUCCESS",
    2022  => "USER_ADD_DISK_FINISHED_FAILURE",
    2023  => "USER_FAILED_ADD_DISK",
    2024  => "USER_RUN_UNLOCK_ENTITY_SCRIPT",
    2025  => "USER_MOVE_IMAGE_GROUP_FAILED_TO_DELETE_SRC_IMAGE",
    2026  => "USER_MOVE_IMAGE_GROUP_FAILED_TO_DELETE_DST_IMAGE",
    2027  => "USER_IMPORT_IMAGE",
    2028  => "USER_IMPORT_IMAGE_FINISHED_SUCCESS",
    2029  => "USER_IMPORT_IMAGE_FINISHED_FAILURE",
    2030  => "USER_EXPORT_IMAGE",
    2031  => "USER_EXPORT_IMAGE_FINISHED_SUCCESS",
    2032  => "USER_EXPORT_IMAGE_FINISHED_FAILURE",
    2033  => "HOT_SET_NUMBER_OF_CPUS",
    2034  => "FAILED_HOT_SET_NUMBER_OF_CPUS",
    2035  => "USER_ISCSI_BOND_HOST_RESTART_WARNING",
    2036  => "ADD_DISK_INTERNAL",
    2037  => "ADD_DISK_INTERNAL_FAILURE",
    2038  => "USER_REMOVE_DISK_INITIATED",
    2039  => "HOT_SET_MEMORY",
    2040  => "FAILED_HOT_SET_MEMORY",
    2041  => "DISK_PREALLOCATION_FAILED",
    2042  => "USER_FINISHED_REMOVE_DISK_ATTACHED_TO_VMS",
    2043  => "USER_FINISHED_REMOVE_DISK_ATTACHED_TO_VMS_NO_DOMAIN",
    2044  => "USER_REMOVE_DISK_ATTACHED_TO_VMS_INITIATED",
    2045  => "USER_COPY_IMAGE_GROUP_FAILED_TO_DELETE_DST_IMAGE",
    2046  => "MEMORY_HOT_UNPLUG_SUCCESSFULLY_REQUESTED",
    2047  => "MEMORY_HOT_UNPLUG_FAILED",
    2048  => "FAILED_HOT_SET_MEMORY_NOT_DIVIDABLE",
    2049  => "MEMORY_HOT_UNPLUG_SUCCESSFULLY_REQUESTED_PLUS_MEMORY_INFO",
    2050  => "NO_MEMORY_DEVICE_TO_HOT_UNPLUG",
    2051  => "NO_SUITABLE_MEMORY_DEVICE_TO_HOT_UNPLUG",
    3000  => "USER_ADD_QUOTA",
    3001  => "USER_FAILED_ADD_QUOTA",
    3002  => "USER_UPDATE_QUOTA",
    3003  => "USER_FAILED_UPDATE_QUOTA",
    3004  => "USER_DELETE_QUOTA",
    3005  => "USER_FAILED_DELETE_QUOTA",
    3006  => "USER_EXCEEDED_QUOTA_CLUSTER_GRACE_LIMIT",
    3007  => "USER_EXCEEDED_QUOTA_CLUSTER_LIMIT",
    3008  => "USER_EXCEEDED_QUOTA_CLUSTER_THRESHOLD",
    3009  => "USER_EXCEEDED_QUOTA_STORAGE_GRACE_LIMIT",
    3010  => "USER_EXCEEDED_QUOTA_STORAGE_LIMIT",
    3011  => "USER_EXCEEDED_QUOTA_STORAGE_THRESHOLD",
    3012  => "QUOTA_STORAGE_RESIZE_LOWER_THEN_CONSUMPTION",
    3013  => "MISSING_QUOTA_STORAGE_PARAMETERS_PERMISSIVE_MODE",
    3014  => "MISSING_QUOTA_CLUSTER_PARAMETERS_PERMISSIVE_MODE",
    3015  => "USER_EXCEEDED_QUOTA_CLUSTER_GRACE_LIMIT_PERMISSIVE_MODE",
    3016  => "USER_EXCEEDED_QUOTA_STORAGE_GRACE_LIMIT_PERMISSIVE_MODE",
    3017  => "USER_IMPORT_IMAGE_AS_TEMPLATE",
    3018  => "USER_IMPORT_IMAGE_AS_TEMPLATE_FINISHED_SUCCESS",
    3019  => "USER_IMPORT_IMAGE_AS_TEMPLATE_FINISHED_FAILURE",
    9000  => "HOST_ALERT_FENCING_IS_NOT_CONFIGURED",
    9001  => "HOST_ALERT_FENCING_TEST_FAILED",
    9002  => "HOST_ALERT_FENCING_OPERATION_FAILED",
    9003  => "HOST_ALERT_FENCING_OPERATION_SKIPPED",
    9004  => "HOST_ALERT_FENCING_NO_PROXY_HOST",
    9005  => "VDS_ALERT_FENCE_STATUS_VERIFICATION_FAILED",
    9006  => "CANNOT_HIBERNATE_RUNNING_VMS_AFTER_CLUSTER_CPU_UPGRADE",
    9007  => "VDS_ALERT_SECONDARY_AGENT_USED_FOR_FENCE_OPERATION",
    9008  => "VDS_HOST_NOT_RESPONDING_CONNECTING",
    9009  => "VDS_ALERT_PM_HEALTH_CHECK_FENCE_AGENT_NON_RESPONSIVE",
    9010  => "VDS_ALERT_PM_HEALTH_CHECK_START_MIGHT_FAIL",
    9011  => "VDS_ALERT_PM_HEALTH_CHECK_STOP_MIGHT_FAIL",
    9012  => "VDS_ALERT_PM_HEALTH_CHECK_RESTART_MIGHT_FAIL",
    9013  => "VDS_ALERT_FENCE_OPERATION_SKIPPED_BROKEN_CONNECTIVITY",
    9014  => "VDS_ALERT_NOT_RESTARTED_DUE_TO_POLICY",
    9015  => "VDS_ALERT_FENCE_DISABLED_BY_CLUSTER_POLICY",
    9016  => "FENCE_DISABLED_IN_CLUSTER_POLICY",
    9017  => "FENCE_OPERATION_STARTED",
    9018  => "FENCE_OPERATION_SUCCEEDED",
    9019  => "FENCE_OPERATION_FAILED",
    9020  => "FENCE_OPERATION_USING_AGENT_AND_PROXY_STARTED",
    9021  => "FENCE_OPERATION_USING_AGENT_AND_PROXY_FAILED",
    9022  => "ENGINE_NO_FULL_BACKUP",
    9023  => "ENGINE_NO_WARM_BACKUP",
    9024  => "ENGINE_BACKUP_STARTED",
    9025  => "ENGINE_BACKUP_COMPLETED",
    9026  => "ENGINE_BACKUP_FAILED",
    9027  => "VDS_HOST_NOT_RESPONDING",
    9500  => "TASK_STOPPING_ASYNC_TASK",
    9501  => "TASK_CLEARING_ASYNC_TASK",
    9502  => "VDS_ACTIVATE_ASYNC",
    9503  => "VDS_ACTIVATE_FAILED_ASYNC",
    9504  => "STORAGE_ACTIVATE_ASYNC",
    9505  => "USER_ACTIVATED_STORAGE_DOMAIN_ASYNC",
    9506  => "USER_ACTIVATE_STORAGE_DOMAIN_FAILED_ASYNC",
    9600  => "IMPORTEXPORT_IMPORT_VM_INVALID_INTERFACES",
    9601  => "VDS_SET_NON_OPERATIONAL_VM_NETWORK_IS_BRIDGELESS",
    9602  => "HA_VM_FAILED",
    9603  => "HA_VM_RESTART_FAILED",
    9604  => "EMULATED_MACHINES_INCOMPATIBLE_WITH_CLUSTER",
    9605  => "EXCEEDED_MAXIMUM_NUM_OF_RESTART_HA_VM_ATTEMPTS",
    9606  => "IMPORTEXPORT_SNAPSHOT_VM_INVALID_INTERFACES",
    9607  => "ADD_VM_FROM_SNAPSHOT_INVALID_INTERFACES",
    9608  => "RNG_SOURCES_INCOMPATIBLE_WITH_CLUSTER",
    9609  => "EMULATED_MACHINES_INCOMPATIBLE_WITH_CLUSTER_LEVEL",
    9610  => "MIXING_RHEL_VERSIONS_IN_CLUSTER",
    9611  => "COLD_REBOOT_VM_DOWN",
    9612  => "COLD_REBOOT_FAILED",
    9613  => "EXCEEDED_MAXIMUM_NUM_OF_COLD_REBOOT_VM_ATTEMPTS",
    9700  => "DWH_STARTED",
    9701  => "DWH_STOPPED",
    9704  => "DWH_ERROR",
    9801  => "EXTERNAL_EVENT_NORMAL",
    9802  => "EXTERNAL_EVENT_WARNING",
    9803  => "EXTERNAL_EVENT_ERROR",
    9804  => "EXTERNAL_ALERT",
    9901  => "WATCHDOG_EVENT",
    9910  => "USER_ADD_CLUSTER_POLICY",
    9911  => "USER_FAILED_TO_ADD_CLUSTER_POLICY",
    9912  => "USER_UPDATE_CLUSTER_POLICY",
    9913  => "USER_FAILED_TO_UPDATE_CLUSTER_POLICY",
    9914  => "USER_REMOVE_CLUSTER_POLICY",
    9915  => "USER_FAILED_TO_REMOVE_CLUSTER_POLICY",
    9920  => "FAILED_TO_CONNECT_TO_SCHEDULER_PROXY",
    10000 => "VDS_UNTRUSTED",
    10001 => "USER_UPDATE_VM_FROM_TRUSTED_TO_UNTRUSTED",
    10002 => "USER_UPDATE_VM_FROM_UNTRUSTED_TO_TRUSTED",
    10003 => "IMPORTEXPORT_IMPORT_VM_FROM_TRUSTED_TO_UNTRUSTED",
    10004 => "IMPORTEXPORT_IMPORT_VM_FROM_UNTRUSTED_TO_TRUSTED",
    10005 => "USER_ADD_VM_FROM_TRUSTED_TO_UNTRUSTED",
    10006 => "USER_ADD_VM_FROM_UNTRUSTED_TO_TRUSTED",
    10007 => "IMPORTEXPORT_IMPORT_TEMPLATE_FROM_TRUSTED_TO_UNTRUSTED",
    10008 => "IMPORTEXPORT_IMPORT_TEMPLATE_FROM_UNTRUSTED_TO_TRUSTED",
    10009 => "USER_ADD_VM_TEMPLATE_FROM_TRUSTED_TO_UNTRUSTED",
    10010 => "USER_ADD_VM_TEMPLATE_FROM_UNTRUSTED_TO_TRUSTED",
    10011 => "USER_UPDATE_VM_TEMPLATE_FROM_TRUSTED_TO_UNTRUSTED",
    10012 => "USER_UPDATE_VM_TEMPLATE_FROM_UNTRUSTED_TO_TRUSTED",
    10013 => "IMPORTEXPORT_GET_EXTERNAL_VMS_NOT_IN_DOWN_STATUS",
    10100 => "USER_ADDED_NETWORK_QOS",
    10101 => "USER_FAILED_TO_ADD_NETWORK_QOS",
    10102 => "USER_REMOVED_NETWORK_QOS",
    10103 => "USER_FAILED_TO_REMOVE_NETWORK_QOS",
    10104 => "USER_UPDATED_NETWORK_QOS",
    10105 => "USER_FAILED_TO_UPDATE_NETWORK_QOS",
    10110 => "USER_ADDED_QOS",
    10111 => "USER_FAILED_TO_ADD_QOS",
    10112 => "USER_REMOVED_QOS",
    10113 => "USER_FAILED_TO_REMOVE_QOS",
    10114 => "USER_UPDATED_QOS",
    10115 => "USER_FAILED_TO_UPDATE_QOS",
    10120 => "USER_ADDED_DISK_PROFILE",
    10121 => "USER_FAILED_TO_ADD_DISK_PROFILE",
    10122 => "USER_REMOVED_DISK_PROFILE",
    10123 => "USER_FAILED_TO_REMOVE_DISK_PROFILE",
    10124 => "USER_UPDATED_DISK_PROFILE",
    10125 => "USER_FAILED_TO_UPDATE_DISK_PROFILE",
    10130 => "USER_ADDED_CPU_PROFILE",
    10131 => "USER_FAILED_TO_ADD_CPU_PROFILE",
    10132 => "USER_REMOVED_CPU_PROFILE",
    10133 => "USER_FAILED_TO_REMOVE_CPU_PROFILE",
    10134 => "USER_UPDATED_CPU_PROFILE",
    10135 => "USER_FAILED_TO_UPDATE_CPU_PROFILE",
    10200 => "USER_UPDATED_MOM_POLICIES",
    10201 => "USER_FAILED_TO_UPDATE_MOM_POLICIES",
    10250 => "PM_POLICY_UP_TO_MAINTENANCE",
    10251 => "PM_POLICY_MAINTENANCE_TO_DOWN",
    10252 => "PM_POLICY_TO_UP",
    10300 => "CLUSTER_ALERT_HA_RESERVATION",
    10301 => "CLUSTER_ALERT_HA_RESERVATION_DOWN",
    10350 => "USER_ADDED_AFFINITY_GROUP",
    10351 => "USER_FAILED_TO_ADD_AFFINITY_GROUP",
    10352 => "USER_UPDATED_AFFINITY_GROUP",
    10353 => "USER_FAILED_TO_UPDATE_AFFINITY_GROUP",
    10354 => "USER_REMOVED_AFFINITY_GROUP",
    10355 => "USER_FAILED_TO_REMOVE_AFFINITY_GROUP",
    10356 => "VM_TO_HOST_CONFLICT_IN_ENFORCING_POSITIVE_AND_NEGATIVE_AFFINITY",
    10357 => "VM_TO_HOST_CONFLICT_IN_POSITIVE_AND_NEGATIVE_AFFINITY",
    10358 => "VM_TO_HOST_CONFLICTS_POSITIVE_VM_TO_VM_AFFINITY",
    10359 => "VM_TO_HOST_CONFLICTS_NEGATIVE_VM_TO_VM_AFFINITY",
    10360 => "NON_INTERSECTING_POSITIVE_HOSTS_AFFINITY_CONFLICTS",
    10361 => "VM_TO_VM_AFFINITY_CONFLICTS",
    10380 => "USER_ADDED_AFFINITY_LABEL",
    10381 => "USER_FAILED_TO_ADD_AFFINITY_LABEL",
    10382 => "USER_UPDATED_AFFINITY_LABEL",
    10383 => "USER_FAILED_TO_UPDATE_AFFINITY_LABEL",
    10384 => "USER_REMOVED_AFFINITY_LABEL",
    10385 => "USER_FAILED_TO_REMOVE_AFFINITY_LABEL",
    10400 => "ISCSI_BOND_ADD_SUCCESS",
    10401 => "ISCSI_BOND_ADD_FAILED",
    10402 => "ISCSI_BOND_EDIT_SUCCESS",
    10403 => "ISCSI_BOND_EDIT_FAILED",
    10404 => "ISCSI_BOND_REMOVE_SUCCESS",
    10405 => "ISCSI_BOND_REMOVE_FAILED",
    10406 => "ISCSI_BOND_EDIT_SUCCESS_WITH_WARNING",
    10407 => "ISCSI_BOND_ADD_SUCCESS_WITH_WARNING",
    10450 => "USER_SET_HOSTED_ENGINE_MAINTENANCE",
    10451 => "USER_FAILED_TO_SET_HOSTED_ENGINE_MAINTENANCE",
    10452 => "USER_HOST_MAINTENANCE",
    10453 => "USER_HOST_MAINTENANCE",
    10454 => "VDS_ACTIVATE_MANUAL_HA",
    10455 => "VDS_ACTIVATE_MANUAL_HA_ASYNC",
    10456 => "HOSTED_ENGINE_VM_IMPORT_SUCCEEDED",
    10457 => "HOSTED_ENGINE_VM_IMPORT_FAILED",
    10458 => "HOSTED_ENGINE_SD_NOT_EXIT",
    10459 => "HOSTED_ENGINE_SD_NOT_ACTIVE",
    10460 => "HOSTED_ENGINE_DOMAIN_IMPORT_SUCCEEDED",
    10461 => "HOSTED_ENGINE_DOMAIN_IMPORT_FAILED",
    10500 => "EXTERNAL_SCHEDULER_PLUGIN_ERROR",
    10501 => "EXTERNAL_SCHEDULER_ERROR",
    10550 => "VM_SLA_POLICY_CPU",
    10551 => "VM_SLA_POLICY_STORAGE",
    10552 => "VM_SLA_POLICY_CPU_STORAGE",
    10553 => "FAILED_VM_SLA_POLICY",
    10600 => "USER_REMOVE_AUDIT_LOG",
    10601 => "USER_REMOVE_AUDIT_LOG_FAILED",
    10602 => "USER_CLEAR_ALL_AUDIT_LOG",
    10603 => "USER_CLEAR_ALL_AUDIT_LOG_FAILED",
    10604 => "USER_DISPLAY_ALL_AUDIT_LOG",
    10605 => "USER_DISPLAY_ALL_AUDIT_LOG_FAILED",
    10700 => "MAC_POOL_ADD_SUCCESS",
    10701 => "MAC_POOL_ADD_FAILED",
    10702 => "MAC_POOL_EDIT_SUCCESS",
    10703 => "MAC_POOL_EDIT_FAILED",
    10704 => "MAC_POOL_REMOVE_SUCCESS",
    10705 => "MAC_POOL_REMOVE_FAILED",
    10750 => "CINDER_PROVIDER_ERROR",
    10751 => "CINDER_DISK_CONNECTION_FAILURE",
    10752 => "CINDER_DISK_CONNECTION_VOLUME_DRIVER_UNSUPPORTED",
    10753 => "USER_FINISHED_FAILED_REMOVE_CINDER_DISK",
    10754 => "USER_ADDED_LIBVIRT_SECRET",
    10755 => "USER_FAILED_TO_ADD_LIBVIRT_SECRET",
    10756 => "USER_UPDATE_LIBVIRT_SECRET",
    10757 => "USER_FAILED_TO_UPDATE_LIBVIRT_SECRET",
    10758 => "USER_REMOVED_LIBVIRT_SECRET",
    10759 => "USER_FAILED_TO_REMOVE_LIBVIRT_SECRET",
    10760 => "FAILED_TO_REGISTER_LIBVIRT_SECRET",
    10761 => "FAILED_TO_UNREGISTER_LIBVIRT_SECRET",
    10762 => "FAILED_TO_REGISTER_LIBVIRT_SECRET_ON_VDS",
    10763 => "NO_LIBRBD_PACKAGE_AVAILABLE_ON_VDS",
    10764 => "FAILED_TO_FREEZE_VM",
    10765 => "FAILED_TO_THAW_VM",
    10766 => "FREEZE_VM_INITIATED",
    10767 => "FREEZE_VM_SUCCESS",
    10768 => "THAW_VM_SUCCESS",
    10769 => "USER_FAILED_TO_FREEZE_VM",
    10770 => "USER_FAILED_TO_THAW_VM",
    10771 => "VDS_CANNOT_CONNECT_TO_GLUSTERFS",
    10780 => "AFFINITY_RULES_ENFORCEMENT_MANAGER_START",
    10781 => "AFFINITY_RULES_ENFORCEMENT_MANAGER_INTERVAL_REACHED",
    10800 => "VM_ADD_HOST_DEVICES",
    10801 => "VM_REMOVE_HOST_DEVICES",
    10802 => "VDS_BROKER_COMMAND_FAILURE",
    10803 => "IRS_BROKER_COMMAND_FAILURE",
    10804 => "VDS_UNKNOWN_HOST",
    10810 => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_UP_REPORTING_HOSTS",
    10811 => "SYSTEM_CHANGE_STORAGE_POOL_STATUS_NON_RESPONSIVE_NO_REPORTING_HOSTS",
    10812 => "STORAGE_POOL_LOWER_THAN_ENGINE_HIGHEST_CLUSTER_LEVEL",
    10900 => "HOST_SYNC_ALL_NETWORKS_FAILED",
    10901 => "HOST_SYNC_ALL_NETWORKS_FINISHED",
    10902 => "PERSIST_HOST_SETUP_NETWORK_ON_HOST",
    10903 => "PERSIST_SETUP_NETWORK_ON_HOST_FINISHED",
    10904 => "PERSIST_SETUP_NETWORK_ON_HOST_FAILED",
    10910 => "NETWORK_REMOVE_NIC_FILTER_PARAMETER",
    10911 => "NETWORK_REMOVE_NIC_FILTER_PARAMETER_FAILED",
    10912 => "NETWORK_ADD_NIC_FILTER_PARAMETER",
    10913 => "NETWORK_ADD_NIC_FILTER_PARAMETER_FAILED",
    10914 => "NETWORK_UPDATE_NIC_FILTER_PARAMETER",
    10915 => "NETWORK_UPDATE_NIC_FILTER_PARAMETER_FAILED",
    10916 => "MAC_ADDRESS_HAD_TO_BE_REALLOCATED",
    10917 => "MAC_ADDRESS_VIOLATES_NO_DUPLICATES_SETTING",
    10918 => "MAC_ADDRESS_COULDNT_BE_REALLOCATED",
    11000 => "USER_ADD_EXTERNAL_JOB",
    11001 => "USER_ADD_EXTERNAL_JOB_FAILED",
    11500 => "FAULTY_MULTIPATHS_ON_HOST",
    11501 => "NO_FAULTY_MULTIPATHS_ON_HOST",
    11502 => "MULTIPATH_DEVICES_WITHOUT_VALID_PATHS_ON_HOST",
    12000 => "MIGRATION_REASON_AFFINITY_ENFORCEMENT",
    12001 => "MIGRATION_REASON_LOAD_BALANCING",
    12002 => "MIGRATION_REASON_HOST_IN_MAINTENANCE",
    12003 => "VM_MIGRATION_NOT_ALL_VM_NICS_WERE_PLUGGED_BACK",
    12004 => "VM_MIGRATION_PLUGGING_VM_NICS_FAILED",
    12005 => "CLUSTER_CANNOT_UPDATE_VM_COMPATIBILITY_VERSION",
    13000 => "DEPRECATED_API",
    13001 => "DEPRECATED_IPTABLES_FIREWALL"
  }.freeze
end
